import 'dart:io';

import 'package:dartz/dartz.dart';
import 'package:aadhaar_offline_ekyc/api/aadhaar_model.dart';
import 'package:aadhaar_offline_ekyc/controllers/aadhaar_sdk_controller.dart';
import 'package:get/get.dart';

import 'failure.dart';

class ApiService extends GetConnect {
  @override
  void onInit() {
    httpClient.timeout = const Duration(seconds: 60);
  }

  Future<Either<Failure, String>> authorize(
      String clientId, String clientSecret) async {
    var body = 'client_id=$clientId&client_secret=$clientSecret';
    final response = await post(
        '${Get.find<AadhaarSdkController>().baseUrl}/authorize', body,
        contentType: 'application/x-www-form-urlencoded');
    if (response.status.hasError) {
      return Left(ServerFailure(
          response.statusText ?? "Error while getting data",
          response.statusCode ?? 404));
    } else {
      if (response.bodyString != null) {
        return Right(response.bodyString!);
      }
      return const Left(ServerFailure("Error while getting data", 404));
    }
  }

  Future<Either<Failure, AadhaarModel>> uploadXml(
      String clientId, String clientSecret, String shareCode, File file,
      {File? selfie}) async {
    MultipartFile f = MultipartFile(file, filename: 'xml_file');

    FormData form = FormData({
      'client_id': clientId,
      'shared_passcode': shareCode,
      'xml_file': f,
    });
    if (selfie != null) {
      MultipartFile selfieFile = MultipartFile(selfie, filename: 'selfie_file');
      form = FormData({
        'client_id': clientId,
        'shared_passcode': shareCode,
        'xml_file': f,
        'selfie_file': selfieFile,
      });
    }
    final response = await post(
      '${Get.find<AadhaarSdkController>().baseUrl}/uploadokycxml',
      form,
      headers: {"x-api-key": clientSecret},
      contentType: "multipart/form-data",
    );
    if (response.status.hasError) {
      return Left(ServerFailure(
          response.bodyString ?? "Error while getting data",
          response.statusCode ?? 404));
    } else {
      if (response.bodyString != null) {
        return Right(AadhaarModel.fromRawJson(response.bodyString!));
      }
      return const Left(ServerFailure("Error while getting data", 404));
    }
  }
}
