import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:isolate';
import 'dart:typed_data';

import 'package:camera/camera.dart';
import 'package:aadhaar_offline_ekyc/api/aadhaar_model.dart';
import 'package:aadhaar_offline_ekyc/controllers/aadhaar_sdk_controller.dart';
import 'package:aadhaar_offline_ekyc/utils/aadhaar_number_validator.dart';
import 'package:aadhaar_offline_ekyc/utils/toast_util.dart';
import 'package:aadhaar_offline_ekyc/utils/utils.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_inappwebview/flutter_inappwebview.dart';
import 'package:flutter_keyboard_visibility/flutter_keyboard_visibility.dart';
import 'package:get/get.dart';
import 'package:path_provider/path_provider.dart';

import '../api/api_service.dart';
import '../offline_aadhaar_sdk.dart';
import '../pages/camera_page.dart';

void timer(SendPort isolateToMainStream) {
  ReceivePort mainToIsolateStream = ReceivePort();

  isolateToMainStream.send(mainToIsolateStream.sendPort);

  String captchaData = "";
  bool checkCaptcha = false;
  bool checkCaptchaError = false;
  bool checkOTP = false;
  int captchaTime = 0;
  int captchaErrorTime = 0;
  int otpTime = 0;

  mainToIsolateStream.listen((data) {
    if (data == TimerStatus.captchaTimer) {
      captchaTime = DateTime.now().millisecondsSinceEpoch;
      checkCaptcha = true;
    } else if (data == TimerStatus.captchaErrorTimer) {
      captchaErrorTime = DateTime.now().millisecondsSinceEpoch;
      checkCaptchaError = true;
    } else if (data == TimerStatus.otpTimer) {
      otpTime = DateTime.now().millisecondsSinceEpoch;
      checkOTP = true;
    } else if (data == TimerStatus.stopCaptchaTimer) {
      checkCaptcha = false;
    } else if (data == TimerStatus.stopCaptchaErrorTimer) {
      checkCaptchaError = false;
    } else if (data == TimerStatus.stopOtpTimer) {
      checkOTP = false;
    } else if (data is String) {
      if (data.contains("base64") && captchaData != data) {
        var base64 = data.split("base64,")[1].split("\"")[0];
        if (base64.endsWith("\\")) {
          base64 = base64.substring(0, base64.length - 1);
        }
        isolateToMainStream.send(base64);
        captchaData = data;
      }
    }
  });
  Timer.periodic(const Duration(seconds: 1), (Timer t) async {
    var currentMilli = DateTime.now().millisecondsSinceEpoch;
    if (checkCaptcha && captchaTime + 2000 < currentMilli) {
      captchaTime = currentMilli;
      isolateToMainStream.send(TimerStatus.captchaTimer);
    }
    if (checkCaptchaError && captchaErrorTime + 1000 < currentMilli) {
      captchaErrorTime = currentMilli;
      isolateToMainStream.send(TimerStatus.captchaErrorTimer);
    }
    if (checkOTP && otpTime + 1500 < currentMilli) {
      otpTime = currentMilli;
      isolateToMainStream.send(TimerStatus.otpTimer);
    }
    isolateToMainStream.send(TimerStatus.mainErrorTimer);
  });
}

class CaptchaData {
  final Image image;

  CaptchaData(this.image);
}

enum TimerStatus {
  captchaTimer,
  stopCaptchaTimer,
  mainErrorTimer,
  captchaErrorTimer,
  stopCaptchaErrorTimer,
  otpTimer,
  stopOtpTimer,
}
enum AadhaarStatus { firstPage, loginPage, applicationPage, shareCodePage }

class WebController extends GetxController {
  Isolate? timerIsolate;
  SendPort? mainToIsolateStream;
  InAppWebViewController? _webController;
  final ApiService _apiService;
  final FocusNode focusNode = FocusNode();
  var showFirstOverlay = false.obs;
  var showSecondOverlay = false.obs;
  var showThirdOverlay = false.obs;
  var showSuccessOverlay = false.obs;
  var showLoader = true.obs;

  var captchaImage = "".obs;
  final AadhaarSdkController _aadhaarSdkController = Get.find();
  final TextEditingController aadhaarController = TextEditingController();
  final TextEditingController captchaController = TextEditingController();
  final TextEditingController otpController = TextEditingController();
  final TextEditingController shareCodeController = TextEditingController();

  var showShareCodeError = false.obs;

  var showLoaderBackground = true.obs;

  WebController(this._apiService);

  void setWebController(InAppWebViewController controller) {
    _webController = controller;
  }

  var currentStatus = AadhaarStatus.firstPage;
  late StreamSubscription<bool> keyboardSubscription;
  bool isKeyboardVisible = false;
  var otpErrorMessage = "Resend OTP Message".obs;

  @override
  void onInit() async {
    mainToIsolateStream = await initIsolate();
    var keyboardVisibilityController = KeyboardVisibilityController();
    keyboardSubscription =
        keyboardVisibilityController.onChange.listen((bool visible) async {
      isKeyboardVisible = visible;
    });
    super.onInit();
  }

  @override
  void dispose() {
    keyboardSubscription.cancel();
    super.dispose();
  }

  void clickLoginButton() {
    _webController?.evaluateJavascript(
        source:
            "document.getElementsByClassName('button_btn__1dRFj')[0].click();");
  }

  void clickDownloadButton() async {
    if (shareCodeController.text.length < 4) {
      showToast("Invalid Share Code. Please enter 4 digit share code.");
      return;
    }
    showLoader.value = true;
    await addDelay(seconds: 2);
    _webController?.evaluateJavascript(
        source:
            "document.getElementsByClassName('button_btn__1dRFj')[1].click();");
    showLoader.value = false;
  }

  void clickSendOtp() {
    Get.focusScope?.unfocus();
    if (!AadhaarNumberValidator.validateAadhaarNumber(aadhaarController.text)) {
      showToast(
          "Invalid aadhaar Number. Please provide the valid aadhaar Number.");
      return;
    } else if (captchaController.text.length < 3) {
      showToast("Invalid Captcha, please enter the captcha again.");
      return;
    }
    showLoader.value = true;
    _webController?.evaluateJavascript(
        source: "document.getElementById('submit-btn').click();");
    checkForCaptchaError();
  }

  void clickReSendOtp() async {
    await _webController?.evaluateJavascript(
        source: "document.getElementById('submit-btn').click();");
    checkForOtp();
  }

  void clickRefreshCaptcha() {
    _webController?.evaluateJavascript(
        source:
            "document.getElementsByClassName('fa fa-refresh icon-style')[0].click();");
    captchaImage.value = "";
  }

  void setTestingAAdhaar(String no) {
    _webController?.evaluateJavascript(
        source:
            "(function() { document.getElementById('uid').value =  '$no'; return 'success'; })();");
  }

  void showAadhaarOverlay() {
    showFirstOverlay.value = true;
    showLoaderBackground.value = false;
    showLoader.value = false;
    // aadhaarController.text = "474992817696";
    // setTestingAAdhaar("474992817696");
    // aadhaarController.text = "684287775658";
    // setTestingAAdhaar("684287775658");
  }

  void checkForCaptcha() {
    mainToIsolateStream!.send(TimerStatus.captchaTimer);
  }

  void checkForCaptchaError() {
    mainToIsolateStream!.send(TimerStatus.captchaErrorTimer);
  }

  void stopCaptchaError() {
    mainToIsolateStream!.send(TimerStatus.stopCaptchaErrorTimer);
  }

  void stopCaptcha() {
    mainToIsolateStream!.send(TimerStatus.stopCaptchaTimer);
  }

  void checkForOtp() {
    mainToIsolateStream!.send(TimerStatus.otpTimer);
  }

  void stopOtp() {
    mainToIsolateStream!.send(TimerStatus.stopOtpTimer);
  }

  void setAadhaarNumber(String value) {
    _webController?.evaluateJavascript(
        source:
            "(function() { document.getElementById('uid').value =  '$value'; return 'success'; })();");
  }

  void setOTP(String value) {
    _webController?.evaluateJavascript(
        source:
            "(function() { document.getElementById('otp').value =  '$value'; return 'success'; })();");
  }

  void setCaptcha(String value) {
    _webController?.evaluateJavascript(
        source:
            "(function() { document.getElementById('captcha').value =  '$value'; return 'success'; })();");
  }

  void clickOtpButton() async {
    if (otpController.text.length < 6) {
      showToast(
          "Invalid OTP . Please enter the OTP received on your aadhaar registered mobile number.");
      return;
    }
    Get.focusScope?.unfocus();
    showLoader.value = true;
    await _webController?.evaluateJavascript(
        source:
            "(function() { document.getElementById('submit').disabled =  false; return 'success'; })();");
    _webController?.evaluateJavascript(
        source: "document.getElementById('submit').click();");
    currentStatus = AadhaarStatus.applicationPage;
  }

  late Completer<SendPort> completer;
  late ReceivePort isolateToMainStream;

  Future<SendPort> initIsolate() async {
    completer = Completer<SendPort>();
    isolateToMainStream = ReceivePort();

    isolateToMainStream.listen((data) async {
      if (data is SendPort) {
        SendPort mainToIsolateStream = data;
        completer.complete(mainToIsolateStream);
      } else {
        if (data is TimerStatus) {
          if (data == TimerStatus.captchaTimer) {
            checkCaptcha();
          } else if (data == TimerStatus.mainErrorTimer) {
            checkMainError();
            checkToastifyError();
          } else if (data == TimerStatus.captchaErrorTimer) {
            checkCaptchaError();
          } else if (data == TimerStatus.otpTimer) {
            checkOtpMessage();
          }
        } else if (data is String) {
          captchaController.text = "";
          captchaImage.value = data;
        }
      }
    });
    timerIsolate = await Isolate.spawn(timer, isolateToMainStream.sendPort);
    return completer.future;
  }

  void checkCaptcha() async {
    var data = await _webController?.evaluateJavascript(
        source:
            "(function() { var item = document.getElementById('captcha_block'); return item.innerHTML; })();");
    mainToIsolateStream!.send(data);
  }

  void stopTimer() {
    if (timerIsolate != null) {
      timerIsolate!.kill(priority: Isolate.immediate);
      timerIsolate = null;
    }
  }

  void checkMainError() async {
    var data = await _webController?.evaluateJavascript(
        source:
            "(function() { var item = document.getElementById('main-message'); return item.innerHTML; })();");
    if (data is String) {
      if (data != "null" && data.isNotEmpty) {
        if (data.toLowerCase().contains("otp")) {
          showToast(
            "Incorrect OTP, please retry eKYC again.",
            callback: () {
              stopSdk();
            },
          );
        } else {
          showToast(
            data,
            callback: () => stopSdk(),
          );
        }
      }
    }
  }

  stopSdk() {
    showLoader.value = false;
    stopTimer();
    Get.delete<WebController>();
    closeSdk();
  }

  var toastMessageTimeStamp = 0;

  void checkToastifyError() async {
    var data = await _webController?.evaluateJavascript(
        source:
            "(function() { var item = document.getElementsByClassName('Toastify__toast-body')[0]; return item.innerHTML; })();");
    if (data is String) {
      if (data != "null" && data.isNotEmpty) {
        if (data.toLowerCase().contains("invalid share code")) {
          if (toastMessageTimeStamp + 2000 <
              DateTime.now().millisecondsSinceEpoch) {
            showLoader.value = false;
            showToast(data);
          }
          toastMessageTimeStamp = DateTime.now().millisecondsSinceEpoch;
        } else {
          showToast(
            data,
            callback: () => stopSdk(),
          );
        }
      }
    }
  }

  void checkCaptchaError() async {
    var data = await _webController?.evaluateJavascript(
        source:
            "(function() { var item = document.getElementById('otp-message'); return item.innerHTML; })();");
    if (data is String) {
      if (data != "null" && data.isNotEmpty) {
        showLoader.value = false;
        showToast(data);
        stopCaptchaError();
        if (data.toLowerCase().contains("success")) {
          checkForOtp();
          stopCaptchaError();
          showSecondOverlay.value = true;
          showFirstOverlay.value = false;
        } else if (data.toLowerCase().contains("mobile number")) {
          _aadhaarSdkController.failureCallback(807);
        } else {
          captchaController.text = "";
        }
      }
    }
  }

  void checkOtpMessage() async {
    var data = await _webController?.evaluateJavascript(
        source:
            "(function() { var item = document.getElementById('resend-otp'); return item.innerHTML; })();");
    if (data is String) {
      if (data != "null" && data.isNotEmpty) {
        if (data.toLowerCase().contains("limit exceeded")) {
          _aadhaarSdkController.failureCallback(806);
          stopOtp();
        }
        var boolData = await _webController?.evaluateJavascript(
            source:
                "(function() { var item = document.getElementById('submit-btn'); return item.disabled; })();");
        if (boolData is bool) {
          if (!boolData) {
            otpErrorMessage.value = "";
            stopOtp();
          } else {
            otpErrorMessage.value = data;
          }
        }
      }
    }
  }

  void openOfflineKyc() async {
    showSecondOverlay.value = false;
    showFirstOverlay.value = false;
    showThirdOverlay.value = true;
    await addDelay(seconds: 6);
    await _webController?.evaluateJavascript(
        source: "document.getElementsByClassName('service-card')[7].click();");
    await addDelay(seconds: 4);

    // await _webController?.evaluateJavascript(
    //     source:
    //         "document.getElementsByClassName('sc-AzgDb fweFII')[0].focus();");
    showLoader.value = false;
  }

  void focusShareCode() async {
    if (!isKeyboardVisible) {
      showLoader.value = true;
      await _webController?.clearFocus();
      await addDelay(seconds: 1);
      await _webController?.evaluateJavascript(
          source:
              "document.getElementsByClassName('sc-AzgDb fweFII')[0].focus();");
      await _webController?.requestFocusNodeHref();
      showLoader.value = false;
    }
  }

  void setShareCode(String data) {
    if (data.isEmpty) {
      showShareCodeError.value = true;
      shareCodeController.text = data;
    } else if (data.length == 4) {
      showShareCodeError.value = false;
      shareCodeController.text = data;
    } else if (shareCodeController.text == data) {
      showShareCodeError.value = true;
    } else {
      showShareCodeError.value = false;
      shareCodeController.text = data;
    }
  }

  Future<File> _write(String base64String) async {
    Uint8List bytes = base64.decode(base64String);
    final Directory directory = await getApplicationDocumentsDirectory();
    final File file = File(
        '${directory.path}/aadhaar${DateTime.now().millisecondsSinceEpoch.toString()}.zip');
    await file.writeAsBytes(bytes);
    return file;
  }

  Future<File?> _read(String path) async {
    try {
      final File file = File(path);
      return file;
    } catch (e) {
      if (kDebugMode) {
        print("Couldn't read file");
      }
    }
    return null;
  }

  void saveZipFile(Uri url) async {
    stopTimer();
    String path = url.path;
    String base64 = path.split("base64,")[1];
    File file = await _write(base64);
    if (kDebugMode) {
      print(file);
    }
    if (_aadhaarSdkController.useFaceMatch) {
      if (cameras!.isNotEmpty) {
        var response = await Get.to(() => const CameraPage());
        if (response != null && response is XFile) {
          File? selfie = await _read(response.path);
          uploadFile(file, selfie: selfie);
        } else {
          uploadFile(file);
        }
      } else {
        showToast("Your Device doesn't support Camera");
        uploadFile(file);
      }
    } else {
      uploadFile(file);
    }
  }

  Rx<AadhaarModel> responseData = AadhaarModel.fromRawJson(
          '{"aadhaar": {"address": {"careOf": "","country": "","district": "","house": "","landmark": "","locality": "","pin": "","postOffice": "","state": "","street": "","subDistrict": "","vtc": ""},"dateOfBirth": "","email": "","gender": "","generatedAt": "","maskedNumber": "","name": "","phone": "","photo": ""}}')
      .obs;
  Rx<File> selfieFile = File("").obs;

  void uploadFile(File xmlFile, {File? selfie}) async {
    if (selfie != null) {
      selfieFile.value = selfie;
    }
    showLoader.value = true;
    var response = await _apiService.uploadXml(
      _aadhaarSdkController.clientId,
      _aadhaarSdkController.clientSecret,
      shareCodeController.text,
      xmlFile,
      selfie: selfie,
    );
    return response.fold((l) {
      showLoader.value = false;
      if (l.message.toLowerCase().contains("host")) {
        showToast(
          "No Internet Available",
          callback: () {
            _aadhaarSdkController.failureCallback(809);
            closeSdk();
          },
        );
      } else if (l.message.toLowerCase().contains("face")) {
        showToast(
          "Error: Face in selfie and face in UIDAI Aadhaar doesn’t match",
          callback: () {
            _aadhaarSdkController.failureCallback(810);
            closeSdk();
          },
        );
      } else {
        showToast(
          l.message,
          callback: () {
            _aadhaarSdkController.failureCallback(809);
            closeSdk();
          },
        );
      }
    }, (r) {
      responseData.value = r;
      showLoader.value = false;
      showSuccessOverlay.value = true;
      if (kDebugMode) {
        print(r);
      }
    });
  }

  void onSuccess() {
    _aadhaarSdkController.successCallback(responseData.value.toRawJson());
  }
}
