import 'dart:convert';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';

abstract class IImageProcessor {
  Widget loadImageFromAssets(
      {required String resId, double? width, double? height, Color? color});

  Widget loadSvgFromAssets(
      {required String resId, double? width, double? height, Color? color});

  Widget loadImageWithUrl(
      {required String imageUrl, double? width, double? height});

  Widget loadNetworkImageWithLoader(
      {required String imageUrl, double? width, double? height});
}

class ImageProcessor extends IImageProcessor {
  static final ImageProcessor _instance = ImageProcessor._internal();

  factory ImageProcessor() {
    return _instance;
  }

  ImageProcessor._internal();

  @override
  Widget loadImageFromAssets(
      {required String resId,
      double? width,
      double? height,
      Color? color,
      BoxFit? fit}) {
    return Image.asset(
      "assets/$resId",
      width: width,
      height: height,
      color: color,
      fit: fit,
      package: 'aadhaar_offline_ekyc',
    );
  }

  @override
  Widget loadImageWithUrl(
      {required String imageUrl, double? width, double? height}) {
    return Image.network(imageUrl, width: width, height: height);
  }

  @override
  Widget loadNetworkImageWithLoader(
      {required String imageUrl, double? width, double? height}) {
    return Stack(
      children: <Widget>[
        const Center(child: CircularProgressIndicator()),
        Center(
          child: Image.network(imageUrl, width: width, height: height),
        ),
      ],
    );
  }

  @override
  Widget loadSvgFromAssets(
      {required String resId, double? width, double? height, Color? color}) {
    return SvgPicture.asset(
      "assets/$resId",
      color: color,
      width: width,
      height: height,
      matchTextDirection: true,
      fit: BoxFit.fitWidth,
    );
  }

  Widget imageFromBase64String(
      String base64String, {double? width, double? height, Color? color}) {
    if (base64String.isEmpty) return Container();
    return Image.memory(
      base64Decode(base64String),
      color: color,
      width: width,
      height: height,
    );
  }
  Widget imageFromFile(File imageFile,
      {double? width, double? height, Color? color}) {
    return Image.file(
      imageFile,
      color: color,
      width: width,
      height: height,
    );
  }
}
