import 'package:aadhaar_offline_ekyc/controllers/aadhaar_sdk_controller.dart';
import 'package:aadhaar_offline_ekyc/pages/first_page.dart';
import 'package:aadhaar_offline_ekyc/utils/custom_colors.dart';
import 'package:camera/camera.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

import 'utils/size_utils.dart';

typedef SuccessCallback = void Function(String response);
typedef FailureCallback = void Function(int failureCode);

void startSdk(BuildContext context, OfflineAadhaarSdk aadhaarSdk) {
  Navigator.push(
    context,
    MaterialPageRoute(builder: (context) => aadhaarSdk),
  );
}

class OfflineAadhaarSdk extends StatefulWidget {
  final String clientId;
  final String clientSecret;
  final String baseUrl;
  final String? imageUrl;
  final bool useFaceMatch;
  final SuccessCallback successCallback;
  final FailureCallback failureCallback;

  const OfflineAadhaarSdk._({
    Key? key,
    required this.clientId,
    required this.clientSecret,
    required this.baseUrl,
    required this.useFaceMatch,
    this.imageUrl,
    required this.successCallback,
    required this.failureCallback,
  }) : super(key: key);

  factory OfflineAadhaarSdk({
    required String clientId,
    required String clientSecret,
    required String baseUrl,
    required bool useFaceMatch,
    String? imageUrl,
    required SuccessCallback successCallback,
    required FailureCallback failureCallback,
  }) {
    return OfflineAadhaarSdk._(
      clientId: clientId,
      clientSecret: clientSecret,
      baseUrl: baseUrl,
      useFaceMatch: useFaceMatch,
      imageUrl: imageUrl,
      successCallback: successCallback,
      failureCallback: failureCallback,
    );
  }

  @override
  _OfflineAadhaarSdkState createState() => _OfflineAadhaarSdkState();
}

List<CameraDescription>? cameras;

class _OfflineAadhaarSdkState extends State<OfflineAadhaarSdk> {
  @override
  void initState() {
    checkForCamera();
    Get.put(
      AadhaarSdkController(
        widget.clientId,
        widget.clientSecret,
        widget.baseUrl,
        widget.imageUrl,
        widget.useFaceMatch,
        widget.successCallback,
        widget.failureCallback,
      ),
    );
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return GetMaterialApp(
      theme: Theme.of(context).copyWith(
        primaryColor: CustomColors().textColor,
        elevatedButtonTheme: ElevatedButtonThemeData(
          style: ElevatedButton.styleFrom(
            primary: CustomColors().textColor,
          ),
        ),
        progressIndicatorTheme:
            ProgressIndicatorThemeData(color: CustomColors().textColor),
        inputDecorationTheme: InputDecorationTheme(
          focusedBorder: UnderlineInputBorder(
              borderSide: BorderSide(color: CustomColors().textColor)),
        ),
        textSelectionTheme: TextSelectionThemeData(
          cursorColor: CustomColors().textColor,
        ),
      ),
      builder: (context, child) {
        initSizeUtils();

        return child!;
      },
      debugShowCheckedModeBanner: false,
      home: FirstPage(),
    );
  }

  void checkForCamera() async {
    WidgetsFlutterBinding.ensureInitialized();
    cameras = await availableCameras();
  }
}
