import 'package:camera/camera.dart';
import 'package:aadhaar_offline_ekyc/image/image_processor.dart';
import 'package:aadhaar_offline_ekyc/utils/size_utils.dart';
import 'package:aadhaar_offline_ekyc/utils/toast_util.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

import '../offline_aadhaar_sdk.dart';
import '../utils/custom_colors.dart';

class CameraPage extends StatefulWidget {
  const CameraPage({Key? key}) : super(key: key);

  @override
  _CameraPageState createState() => _CameraPageState();
}

class _CameraPageState extends State<CameraPage> {
  CameraController? controller;
  bool isFrontCamera = true;
  bool showInstructions = true;

  @override
  void initState() {
    super.initState();
    if (cameras != null) {
      for (int i = 0; i < cameras!.length; i++) {
        if (cameras![i].lensDirection == CameraLensDirection.front) {
          controller = CameraController(cameras![i], ResolutionPreset.max);
          break;
        }
      }
      if (controller == null) {
        showToast("Front Camera not available");
        Get.back();
      }
      controller?.initialize().then((_) {
        if (!mounted) {
          return;
        }
        setState(() {});
      });
    }
  }

  @override
  void dispose() {
    controller?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (controller == null || controller?.value.isInitialized != true) {
      return Container();
    }
    return Scaffold(
      backgroundColor: Colors.white,
      body: SafeArea(
        child: SizedBox(
          height: double.infinity,
          child: Stack(
            children: [
              CameraPreview(controller!),
              Positioned(
                bottom: 0,
                left: 0,
                right: 0,
                child: Container(
                  padding: EdgeInsets.only(
                    bottom: 60.iY,
                    top: 60.iY,
                    left: 30.iX,
                    right: 30.iX,
                  ),
                  color: Colors.white,
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      SizedBox(
                        width: 80.iY,
                      ),
                      InkWell(
                        onTap: () async {
                          var file = await controller!.takePicture();
                          if (kDebugMode) {
                            print("-------------------------${file.path}");
                          }
                          Get.back(result: file);
                        },
                        child: Icon(
                          Icons.camera,
                          size: 140.iY,
                          color: Colors.black,
                        ),
                      ),
                      InkWell(
                        onTap: () {
                          Get.back(result: null);
                        },
                        child: Icon(
                          Icons.close,
                          size: 80.iY,
                          color: Colors.black,
                        ),
                      ),
                    ],
                  ),
                ),
              ),
              showInstructions
                  ? Positioned(
                      left: 0,
                      right: 0,
                      top: 0,
                      bottom: 0,
                      child: Container(
                        padding: EdgeInsets.symmetric(horizontal: 30.iX),
                        color: Colors.white,
                        child: Column(
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: [
                            SizedBox(
                              height: 160.iY,
                            ),
                            Container(
                              width: double.infinity,
                              child: Text(
                                "Instructions for taking selfie",
                                textAlign: TextAlign.center,
                                style: TextStyle(
                                    color: CustomColors().textColor,
                                    fontWeight: FontWeight.bold,
                                    fontSize: 56.iY),
                              ),
                            ),
                            SizedBox(height: 80.iY),
                            Row(
                              mainAxisAlignment: MainAxisAlignment.center,
                              children: [
                                ImageProcessor().loadImageFromAssets(
                                    resId: "icon/10.png", width: 400.iX)
                              ],
                            ),
                            SizedBox(height: 80.iY),
                            Text(
                              "Follow these instructions for faster approval:",
                              style: TextStyle(
                                fontSize: 34.iY,
                                fontWeight: FontWeight.w600,
                                color: CustomColors().textColor,
                              ),
                            ),
                            SizedBox(height: 40.iY),
                            Text(
                              "- Remove your glasses and then take the selfie",
                              style: TextStyle(
                                fontSize: 34.iY,
                                color: CustomColors().textColor,
                              ),
                            ),
                            SizedBox(height: 40.iY),
                            Text(
                              "- Face forward and make sure your eyes are clearly visible",
                              style: TextStyle(
                                fontSize: 34.iY,
                                color: CustomColors().textColor,
                              ),
                            ),
                            SizedBox(height: 40.iY),
                            Text(
                              "- Make sure your face fits well within the selfie",
                              style: TextStyle(
                                fontSize: 34.iY,
                                color: CustomColors().textColor,
                              ),
                            ),
                            Expanded(
                              child: Container(),
                            ),
                            SizedBox(
                              width: double.infinity,
                              child: ElevatedButton(
                                onPressed: () {
                                  setState(() {
                                    showInstructions = false;
                                  });
                                },
                                child: const Text("Continue"),
                              ),
                            ),
                            SizedBox(height: 40.iY),
                          ],
                        ),
                      ),
                    )
                  : Container(),
            ],
          ),
        ),
      ),
    );
  }
}
