import 'package:aadhaar_offline_ekyc/api/api_service.dart';
import 'package:aadhaar_offline_ekyc/controllers/aadhaar_sdk_controller.dart';
import 'package:aadhaar_offline_ekyc/controllers/main_controller.dart';
import 'package:aadhaar_offline_ekyc/image/image_processor.dart';
import 'package:aadhaar_offline_ekyc/utils/close_dialog.dart';
import 'package:aadhaar_offline_ekyc/utils/custom_colors.dart';
import 'package:aadhaar_offline_ekyc/utils/size_utils.dart';
import 'package:aadhaar_offline_ekyc/utils/utils.dart';
import 'package:flutter/material.dart';
import 'package:flutter_inappwebview/flutter_inappwebview.dart';
import 'package:get/get.dart';

class MainPage extends StatelessWidget {
  MainPage({Key? key}) : super(key: key);
  final WebController pageController = Get.put(WebController(ApiService()));

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: () async {
        CloseDialog().showDialog();
        return false;
      },
      child: Scaffold(
        body: Stack(
          children: [
            Focus(
              focusNode: pageController.focusNode,
              canRequestFocus: true,
              descendantsAreFocusable: true,
              child: InAppWebView(
                initialUrlRequest: URLRequest(
                    url: Uri.parse("https://myaadhaar.uidai.gov.in")),
                initialOptions: InAppWebViewGroupOptions(
                  android: AndroidInAppWebViewOptions(
                    useHybridComposition: true,
                  ),
                  crossPlatform: InAppWebViewOptions(
                      supportZoom: false,
                      javaScriptEnabled: true,
                      useOnDownloadStart: true,
                      useShouldOverrideUrlLoading: false,
                      useShouldInterceptAjaxRequest: false),
                ),
                onWebViewCreated: (controller) {
                  pageController.setWebController(controller);
                  controller.addJavaScriptHandler(
                    handlerName: 'keypress',
                    callback: (data) async {
                      if (data.isNotEmpty && data[0] is String) {
                        pageController.setShareCode(data[0]);
                      }
                    },
                  );
                  controller.addJavaScriptHandler(
                    handlerName: 'touchstart',
                    callback: (data) async {
                      pageController.focusShareCode();
                    },
                  );
                },
                onDownloadStart: (controller, url) {
                  pageController.saveZipFile(url);
                },
                onLoadStop: (controller, url) async {
                  if (pageController.currentStatus == AadhaarStatus.firstPage) {
                    pageController.clickLoginButton();
                    pageController.currentStatus = AadhaarStatus.loginPage;
                  } else if (pageController.currentStatus ==
                      AadhaarStatus.loginPage) {
                    pageController.showAadhaarOverlay();
                    pageController.checkForCaptcha();
                  } else if (pageController.currentStatus ==
                      AadhaarStatus.applicationPage) {
                    pageController.openOfflineKyc();
                    controller.evaluateJavascript(
                        source:
                            "document.addEventListener('keyup', (e) => {window.flutter_inappwebview.callHandler('keypress', event.target.value);})");
                    controller.evaluateJavascript(
                        source:
                            "document.addEventListener('touchstart', (e) => {window.flutter_inappwebview.callHandler('touchstart', event);})");
                  }
                },
              ),
            ),
            Obx(() {
              return Visibility(
                  visible: pageController.showFirstOverlay.value,
                  child: _buildFirstOverlay());
            }),
            Obx(() {
              return Visibility(
                  visible: pageController.showSecondOverlay.value,
                  child: _buildOTPOverlay());
            }),
            Obx(() {
              return Visibility(
                  visible: pageController.showThirdOverlay.value,
                  child: _buildThirdOverlay());
            }),
            Obx(() {
              return Visibility(
                  visible: pageController.showLoader.value,
                  child: _buildLoader());
            }),
            Obx(() {
              return Visibility(
                  visible: pageController.showSuccessOverlay.value,
                  child: _buildSuccessDialog());
            }),
          ],
        ),
      ),
    );
  }

  Widget _buildFirstOverlay() {
    return Container(
      color: Colors.black54,
      child: Column(
        children: [
          Flexible(flex: 1, child: Container()),
          Flexible(
            flex: 5,
            child: Container(
              padding: EdgeInsets.symmetric(vertical: 20.iY, horizontal: 40.iX),
              color: Colors.white,
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      Text(
                        "Powered By",
                        style: TextStyle(fontSize: 20.iY),
                      ),
                      SizedBox(
                        width: 10.iX,
                      ),
                      ImageProcessor().loadImageFromAssets(
                        resId: "icon/9.png",
                        width: 150.iX,
                      ),
                    ],
                  ),
                  SizedBox(height: 40.iY),
                  Text(
                    "Enter Aadhaar",
                    style: TextStyle(
                        fontSize: 30.iY, color: CustomColors().textColor),
                  ),
                  Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      ImageProcessor().loadImageFromAssets(
                          resId: "icon/6.png", width: 60.iX),
                      SizedBox(
                        width: 20.iX,
                      ),
                      Expanded(
                        child: TextField(
                          keyboardType: TextInputType.number,
                          controller: pageController.aadhaarController,
                          onChanged: (value) {
                            pageController.setAadhaarNumber(value);
                          },
                        ),
                      ),
                    ],
                  ),
                  SizedBox(height: 20.iY),
                  Row(
                    mainAxisAlignment: MainAxisAlignment.start,
                    children: [
                      ImageProcessor().loadImageFromAssets(
                          resId: "icon/5.png", width: 30.iX),
                      SizedBox(width: 10.iX),
                      Text(
                        "12 digit Aadhaar number",
                        style: TextStyle(
                            fontSize: 24.iY, color: CustomColors().textColor),
                      ),
                    ],
                  ),
                  SizedBox(height: 40.iY),
                  Text(
                    "Security Code (as seen in picture)",
                    style: TextStyle(
                        fontSize: 30.iY, color: CustomColors().textColor),
                  ),
                  Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      ImageProcessor().loadImageFromAssets(
                          resId: "icon/2.png", width: 60.iX),
                      SizedBox(
                        width: 20.iX,
                      ),
                      Flexible(
                        flex: 3,
                        child: TextField(
                          controller: pageController.captchaController,
                          onChanged: (value) {
                            pageController.setCaptcha(value);
                          },
                        ),
                      ),
                      Flexible(
                        flex: 2,
                        child: Obx(() {
                          return Stack(children: [
                            ImageProcessor().imageFromBase64String(
                                pageController.captchaImage.value),
                            Center(
                                child: Visibility(
                              visible:
                                  pageController.captchaImage.value.isEmpty,
                              child: Transform.scale(
                                  scale: 0.5,
                                  child: const CircularProgressIndicator()),
                            ))
                          ]);
                        }),
                      ),
                      InkWell(
                          onTap: () {
                            pageController.clickRefreshCaptcha();
                          },
                          child: const Icon(Icons.refresh)),
                    ],
                  ),
                  SizedBox(height: 50.iY),
                  Text(
                    "I understand and agree to download my KYC data from UIDAI via paperless aadhaar as recommended by UIDAI and that my Aadhaar Number is used only on UIDAI Website and not stored or transferred anywhere .",
                    style: TextStyle(
                        fontSize: 24.iY, color: CustomColors().textColor),
                  ),
                  SizedBox(height: 20.iY),
                  Text(
                    "By Proceeding, I authorize ${Get.find<AadhaarSdkController>().clientId} to collect my masked paperless aadhaar document in share code protected form and share with ${Get.find<AadhaarSdkController>().clientId} for KYC purposes.",
                    style: TextStyle(
                        fontSize: 24.iY, color: CustomColors().textColor),
                  ),
                  const Expanded(child: SizedBox()),
                  Container(
                    padding: EdgeInsets.symmetric(horizontal: 30.iX),
                    width: double.infinity,
                    child: ElevatedButton(
                        onPressed: () {
                          pageController.clickSendOtp();
                        },
                        child: const Text("Submit")),
                  ),
                ],
              ),
            ),
          )
        ],
      ),
    );
  }

  Widget _buildOTPOverlay() {
    return Container(
      color: Colors.black54,
      child: Column(
        children: [
          Flexible(flex: 1, child: Container()),
          Flexible(
            flex: 5,
            child: Container(
              padding: EdgeInsets.symmetric(vertical: 20.iY, horizontal: 40.iX),
              color: Colors.white,
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      Text(
                        "Powered By",
                        style: TextStyle(fontSize: 20.iY),
                      ),
                      SizedBox(
                        width: 10.iX,
                      ),
                      ImageProcessor().loadImageFromAssets(
                        resId: "icon/9.png",
                        width: 150.iX,
                      ),
                    ],
                  ),
                  SizedBox(height: 40.iY),
                  Text(
                    "OTP",
                    style: TextStyle(
                        fontSize: 30.iY, color: CustomColors().textColor),
                  ),
                  Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      ImageProcessor().loadImageFromAssets(
                          resId: "icon/3.png", width: 60.iX),
                      SizedBox(
                        width: 20.iX,
                      ),
                      Expanded(
                        child: TextField(
                          keyboardType: TextInputType.number,
                          maxLength: 6,
                          controller: pageController.otpController,
                          onChanged: (value) {
                            pageController.setOTP(value);
                          },
                        ),
                      ),
                    ],
                  ),
                  SizedBox(height: 20.iY),
                  Row(
                    mainAxisAlignment: MainAxisAlignment.start,
                    children: [
                      ImageProcessor().loadImageFromAssets(
                          resId: "icon/5.png", width: 30.iX),
                      SizedBox(width: 10.iX),
                      Text(
                        "Please enter OTP sent to your linked mobile number",
                        style: TextStyle(
                            fontSize: 24.iY, color: CustomColors().textColor),
                      ),
                    ],
                  ),
                  SizedBox(height: 20.iY),
                  Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      Obx(() {
                        return Text(
                          pageController.otpErrorMessage.value,
                          style: TextStyle(
                              fontSize: 24.iY,
                              color: CustomColors().errorTextColor),
                        );
                      }),
                    ],
                  ),
                  Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      Obx(() {
                        return Visibility(
                            visible:
                                pageController.otpErrorMessage.value.isEmpty,
                            child: ElevatedButton(
                                onPressed: () {
                                  pageController.clickReSendOtp();
                                },
                                child: const Text("Resend OTP")));
                      }),
                    ],
                  ),
                  const Expanded(child: SizedBox()),
                  Container(
                    padding: EdgeInsets.symmetric(horizontal: 30.iX),
                    width: double.infinity,
                    child: ElevatedButton(
                        onPressed: () {
                          pageController.clickOtpButton();
                        },
                        child: const Text("Submit")),
                  ),
                ],
              ),
            ),
          )
        ],
      ),
    );
  }

  Widget _buildLoader() {
    return Obx(() {
      return Container(
        color: pageController.showLoaderBackground.value
            ? Colors.white
            : Colors.black54,
        child: Center(
          child: Container(
              decoration: BoxDecoration(
                  color: Colors.white,
                  borderRadius: BorderRadius.circular(8.0)),
              padding: EdgeInsets.all(40.iX),
              child: const CircularProgressIndicator()),
        ),
      );
    });
  }

  Widget _buildThirdOverlay() {
    return Column(
      children: [
        Flexible(
            flex: 1,
            child: Container(
              color: Colors.black54,
            )),
        Flexible(
          flex: 3,
          child: IgnorePointer(
            child: Container(
              padding: EdgeInsets.symmetric(vertical: 20.iY, horizontal: 40.iX),
              color: Colors.white,
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      Text(
                        "Powered By",
                        style: TextStyle(fontSize: 20.iY),
                      ),
                      SizedBox(
                        width: 10.iX,
                      ),
                      ImageProcessor().loadImageFromAssets(
                        resId: "icon/9.png",
                        width: 150.iX,
                      ),
                    ],
                  ),
                  SizedBox(height: 40.iY),
                  Text(
                    "Share Code (Any 4 digit number)",
                    style: TextStyle(
                        fontSize: 30.iY, color: CustomColors().textColor),
                  ),
                  Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      ImageProcessor().loadImageFromAssets(
                          resId: "icon/4.png", width: 60.iX),
                      SizedBox(
                        width: 20.iX,
                      ),
                      Expanded(
                        child: TextField(
                          controller: pageController.shareCodeController,
                        ),
                      ),
                    ],
                  ),
                  SizedBox(height: 20.iY),
                  Row(
                    mainAxisAlignment: MainAxisAlignment.start,
                    children: [
                      ImageProcessor().loadImageFromAssets(
                          resId: "icon/5.png", width: 30.iX),
                      SizedBox(width: 10.iX),
                      Expanded(
                        child: Text(
                          "Enter any 4 digit share code /password for securing your aadhar document",
                          style: TextStyle(
                              fontSize: 24.iY, color: CustomColors().textColor),
                        ),
                      ),
                    ],
                  ),
                  SizedBox(height: 30.iY),
                  Obx(() {
                    return Visibility(
                      visible: pageController.showShareCodeError.value,
                      child: Row(
                        children: [
                          Expanded(
                            child: Text(
                              "Please enter digits only (0-9)",
                              textAlign: TextAlign.center,
                              style: TextStyle(
                                  fontSize: 24.iY,
                                  color: CustomColors().errorTextColor),
                            ),
                          ),
                        ],
                      ),
                    );
                  }),
                  const Expanded(child: SizedBox()),
                ],
              ),
            ),
          ),
        ),
        Flexible(
          flex: 3,
          child: Container(
            color: Colors.white,
            padding: EdgeInsets.symmetric(
              horizontal: 70.iX,
              vertical: 20.iY,
            ),
            child: Align(
              alignment: Alignment.bottomCenter,
              child: SizedBox(
                width: double.infinity,
                child: ElevatedButton(
                    onPressed: () {
                      pageController.clickDownloadButton();
                    },
                    child: const Text("Submit")),
              ),
            ),
          ),
        ),
      ],
    );
  }

  Widget _buildSuccessDialog() {
    return Container(
      color: Colors.black54,
      child: Center(
        child: Container(
          decoration: BoxDecoration(
              color: Colors.white, borderRadius: BorderRadius.circular(12.0)),
          width: 700.iX,
          padding: EdgeInsets.symmetric(vertical: 20.iY, horizontal: 40.iX),
          child: Column(
            mainAxisSize: MainAxisSize.min,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              SizedBox(height: 20.iY),
              Row(
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  Text(
                    "KYC Successfully Completed",
                    style: TextStyle(
                        fontSize: 32.iY, color: CustomColors().textColor),
                  ),
                ],
              ),
              SizedBox(height: 30.iY),
              Row(
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  Obx(() => ImageProcessor().imageFromBase64String(
                      pageController.responseData.value.aadhaar.photo,
                      height: 300.iY)),
                  Obx(() {
                    if (pageController.responseData.value.facematch != null) {
                      return Container(
                        margin: EdgeInsets.only(left: 50.iX),
                        child: ImageProcessor().imageFromFile(
                            pageController.selfieFile.value,
                            height: 300.iY,
                            width: 200.iX),
                      );
                    }
                    return Container();
                  }),
                ],
              ),
              SizedBox(height: 30.iY),
              Obx(() {
                return Text(
                  pageController.responseData.value.displayModel(),
                  style: TextStyle(fontSize: 30.iY),
                );
              }),
              Row(
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  ElevatedButton(
                      onPressed: () {
                        pageController.onSuccess();
                        closeSdk();
                      },
                      child: const Text("Done")),
                ],
              ),
              SizedBox(height: 20.iY),
            ],
          ),
        ),
      ),
    );
  }
}
